#!/usr/bin/python
# License: GPLv2
"""Utility for providing a DDNS service for DD-WRT and similar routers using
Linode's DNS manager.
"""
import os
import ConfigParser as configparser


# Define this early so we can use it for import errors.
def send_response(status, body):
    """Utility function for sending a CGI response"""
    print "Content-type: text/html"
    print status
    print ""
    print "<html>"
    print "<head><title>DDNS update</title></head>"
    print "<body>"
    print body
    print "</body>"
    print "</html>"


try:
    from linode.api import Api, ApiError
except ImportError:
    send_response("Status: 500 Internal Server Error",
        "linode-python package not installed.  "
        "See https://pypi.python.org/pypi/linode-python/1.0")


def update_domain_resource(fqdn, address, key):
    """Update the Linode DNS manager"""
    api = Api(key=key)
    base_domain = '.'.join(fqdn.split('.')[-2:])
    hostname = fqdn[:-len(base_domain)-1]

    domains_by_name = dict((d['DOMAIN'], d) for d in api.domain_list())
    domain_to_update = domains_by_name[base_domain]
    domain_id = domain_to_update['DOMAINID']
    resources = api.domain_resource_list(DomainID=domain_id)
    hosts_by_name = dict((r['NAME'], r) for r in resources)
    host_to_update = hosts_by_name[hostname]
    resource_id = host_to_update['RESOURCEID']
    # Finally, make the call to update the domain
    api.domain_resource_update(DomainID=domain_id, ResourceID=resource_id,
        Target=address)


def cgi_update():
    """Using the configuration file, update the authenticated user's host names
    to the user's IP address.  Returns a CGI status and an explanatory message
    for the body.
    """
    if os.environ['AUTH_TYPE'] != 'Basic':
        return "Status: 401 Unauthorized", "No authentication given"
    config = configparser.ConfigParser()
    try:
        config.readfp(open(os.path.join(os.path.dirname(__file__), 'ddns.ini')))
    except IOError, error:
        return ("Status: 500 Internal Server Error",
            "Failed to find configuration file; %s" % error)
    try:
        apikey = config.get('ddns', 'key')
    except configparser.Error, error:
        return ("Status: 500 Internal Server Error",
            "Failed to find key in configuration; %s" % error)
    try:
        user = os.environ['REMOTE_USER']
    except KeyError:
        return ("Status: 401 Unauthorized", "No user given")
    try:
        new_address = os.environ['REMOTE_ADDR']
    except KeyError:
        return ("Status: 500 Internal Server Error", "No remote IP address")
    try:
        domains = [d.strip() for d in config.get(user,
            'domains').split(',')]
    except configparser.Error, error:
        return ("Status: 500 Internal Server Error",
            "Failed to find user's domains in configuration; %s" % error)
    try:
        for domain in domains:
            update_domain_resource(domain, new_address, apikey)
    except ApiError, error:
        return ("Status: 500 Internal Server Error",
            "update of %s failed with %s" % (domain, error))

    return ("Status: 200 OK",
        "Updated %s to %s" % (', '.join(domains), new_address))


def main():
    """Runs as a CGI script"""
    status, body = cgi_update()
    send_response(status, body)
    return 0


if __name__ == '__main__':
    main()
